﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms;
using Buddy.Coroutines;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;
using Kupo.Helpers;
using TreeSharp;
using Action = TreeSharp.Action;

namespace Kupo.Rotations
{
    public class Astrologian : KupoRoutine
    {
        //DEVELOPERS REPLACE GetType().Name WITH YOUR CR'S NAME.
        public override string Name
        {
            get { return "Kupo [" + GetType().Name + "]"; }
        }

        public override ClassJobType[] Class
        {
            get { return new[] { ClassJobType.Astrologian }; }
        }

        public override float PullRange
        {
            get { return 20; }
        }


        [Behavior(BehaviorType.Rest)]
        public Composite CreateBasicRest()
        {
            return new PrioritySelector(
                Spell.Cast("Lucid Dreaming", r => Core.Player.CurrentManaPercent <= 50, r => Core.Player),
                DefaultRestBehavior(r => Core.Player.CurrentManaPercent));
        }

        [Behavior(BehaviorType.Pull)]
        public Composite CreateBasicPull()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null, new PrioritySelector(
            new Decorator(ctx => !RoutineManager.IsAnyDisallowed(CapabilityFlags.Movement), new PrioritySelector(
                         CommonBehaviors.MoveToLos(ctx => ctx as BattleCharacter),
                        CommonBehaviors.MoveAndStop(ctx => (ctx as BattleCharacter).Location, PullRange, true, "Moving to unit")
            )),
            Spell.PullCast(s => HighestCombust),
            Spell.PullCast("Malefic")
    )));
        }

        [Behavior(BehaviorType.CombatBuffs)]
        public Composite CreateBasicCombatBuffs()
        {
            return new PrioritySelector(SummonChocobo());
        }

        [Behavior(BehaviorType.PreCombatBuffs)]
        public Composite CreateBasicPreCombatBuffs()
        {
            return new PrioritySelector(
                SummonChocobo()
                );
        }

        [Behavior(BehaviorType.Heal)]
        public Composite CreateBasicHeal()
        {
            return new PrioritySelector(
                //If we have a Enhanced Benefic II and we have Benefic II use it!
                Spell.Cast("Aspected Benefic", r => !Core.Player.HasAura("Aspected Benefic") && Core.Player.CurrentHealthPercent <= 80, r => Core.Player),
                Spell.Cast("Benefic II", r => Core.Player.HasAura("Enhanced Benefic II"), r => Core.Player),
                Spell.Cast("Benefic", r => Core.Player.CurrentHealthPercent <= 40, r => Core.Player),
                Spell.Cast("Essential Dignity", r => Core.Player.CurrentHealthPercent <= 20, r => Core.Player),
                Spell.Cast("Lucid Dreaming", r => Core.Player.CurrentManaPercent <= 50, r => Core.Player)
                );
        }

        private static HashSet<string> auraSet = new HashSet<string>()
        {
            "The Balance" ,
            "The Arrow"   ,
            "The Spear"   ,
            "The Bole"    ,
            "The Ewer"    ,
            "The Spire"   ,
        };

        private static HashSet<string> auraDrawnSet = new HashSet<string>()
        {
            "Balance Drawn",
            "Arrow Drawn",
            "Spear Drawn",
            "Bole Drawn",
            "Ewer Drawn",
            "Spire Drawn",
        };
        private static FrameCachedValue<bool> hasAura = new FrameCachedValue<bool>(() =>
        {
            var player = Core.Player;
            var auras = player.Auras.AuraList;
            foreach (var aura in auras)
            {
                if (auraSet.Contains(aura.Name))
                    return true;
            }

            return false;
        });

        private static FrameCachedValue<bool> hasDrawnAura = new FrameCachedValue<bool>(() =>
        {
            var player = Core.Player;
            var auras = player.Auras.AuraList;
            foreach (var aura in auras)
            {
                if (auraDrawnSet.Contains(aura.Name))
                    return true;
            }

            return false;
        });



        private static string HighestCombust
        {
            get
            {
                if (ActionManager.HasSpell("Combust III"))
                    return "Combust III";
                if (ActionManager.HasSpell("Combust II"))
                    return "Combust II";

                return "Combust";
            }
        }

        private readonly SpellData Draw = DataManager.GetSpellData(3590);
        private async Task<bool> DrawThenPlay()
        {
            if (Draw.Charges >= 1)
            {
                if (await Coroutine.Wait(5000, () => ActionManager.DoAction("Draw", null)))
                {
                    if (await Coroutine.Wait(5000, () => hasDrawnAura))
                    {
                        if (await Coroutine.Wait(5000, () => ActionManager.DoAction("Play", null)))
                        {

                            if (await Coroutine.Wait(5000, () => hasAura))
                            {
                                return true;
                            }
                        }
                    }

                }
            }


            return false;
        }


        private async Task<bool> Play()
        {
            if (await Coroutine.Wait(5000, () => hasDrawnAura))
            {
                if (await Coroutine.Wait(5000, () => ActionManager.DoAction("Play", null)))
                {

                    if (await Coroutine.Wait(5000, () => hasAura))
                    {
                        return true;
                    }
                }
            }
            return false;

        }

        [Behavior(BehaviorType.Combat)]
        public Composite CreateBasicCombat()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null,
                    new PrioritySelector(
                        new Decorator(ctx => !RoutineManager.IsAnyDisallowed(CapabilityFlags.Movement), new PrioritySelector(
                        CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
                        CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, PullRange, true, "Moving to unit")
                        )),

                        //Spell.Cast("Draw",r=> !hasAura && !hasDrawnAura),
                        //Spell.Cast("Play", r => hasDrawnAura),
                        new Decorator(r => !hasAura && !hasDrawnAura, new ActionRunCoroutine(r => DrawThenPlay())),
                        new Decorator(r => hasDrawnAura, new ActionRunCoroutine(r => Play())),

                        Spell.Apply(s => HighestCombust, null, r => Core.Player.CurrentTarget),
                        Spell.Cast("Malefic"),
                        Spell.Cast("Astrodyne")
                        )));
        }
    }
}