﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using System.Windows.Forms.VisualStyles;
using Buddy.Coroutines;
using Clio.Utilities;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Managers;
using ff14bot.Objects;
using Kupo.Helpers;
using Newtonsoft.Json;
using TreeSharp;

namespace Kupo.Rotations
{

    internal class Machinist : KupoRoutine
    {
        //DEVELOPERS REPLACE GetType().Name WITH YOUR CR'S NAME.
        public override string Name
        {
            get { return "Kupo [" + GetType().Name + "]"; }
        }

        public override ClassJobType[] Class
        {
            get { return new[] { ClassJobType.Machinist }; }
        }

        public override float PullRange
        {
            get { return 20f; }
        }

        [Behavior(BehaviorType.CombatBuffs)]
        public Composite CreateBasicCombatBuffs()
        {
            return new PrioritySelector(SummonChocobo());
        }
        [Behavior(BehaviorType.PreCombatBuffs)]
        public Composite CreateBasicPreCombatBuffs()
        {
            return new PrioritySelector(
                SummonChocobo()
            );
        }

        [Behavior(BehaviorType.Rest)]
        public Composite CreateBasicRest()
        {
            return DefaultRestBehavior(r => Core.Player.CurrentTPPercent);
        }


        [Behavior(BehaviorType.Heal)]
        public Composite CreateBasicHeal()
        {
            return new PrioritySelector(
                Spell.Cast("Second Wind", r => Core.Player.CurrentHealthPercent < 40, r => Core.Player)
            );
        }

        [Behavior(BehaviorType.Pull)]
        public Composite CreateBasicPull()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null,
                    new PrioritySelector(
                        new Decorator(ctx => !RoutineManager.IsAnyDisallowed(CapabilityFlags.Movement), new PrioritySelector(
                        CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
                        CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, ctx => Core.Player.CombatReach + PullRange + (ctx as GameObject).CombatReach, true, "Moving to unit")
                        )),


                        Spell.Cast("Clean Shot", r => ActionManager.LastSpell.Name == "Slug Shot"),
                        Spell.Cast("Slug Shot", r => ActionManager.LastSpell.Name == "Split Shot"),
                        Spell.Cast("Split Shot")
                        )));
        }


        #region spell data for some spells
        private readonly SpellData Hypercharge = DataManager.GetSpellData(17209);


        private readonly SpellData Drill = DataManager.GetSpellData(16498);
        private readonly SpellData HotShot = DataManager.GetSpellData(2872);
        private readonly SpellData Reassemble = DataManager.GetSpellData(2876);

            
        #endregion

        private bool shouldReassemble
        {
            get
            {
                var spell = HotShot;
                if (ActionManager.HasSpell("Drill"))
                    spell = Drill;


                return spell.Cooldown.TotalSeconds <= 0;
            }
        }

        [Behavior(BehaviorType.Combat)]
        public Composite CreateBasicCombat()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null,
                    new PrioritySelector(
                        new Decorator(ctx => !RoutineManager.IsAnyDisallowed(CapabilityFlags.Movement), new PrioritySelector(
                        CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
                        CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, ctx => Core.Player.CombatReach + PullRange + (ctx as GameObject).CombatReach, true, "Moving to unit")
                        )),


                        Spell.Cast("Reassemble", r => shouldReassemble),

                        Spell.Cast("Drill"),
                        Spell.Cast("Heat Blast"),
                        Spell.Cast("Hot Shot"),
                        Spell.Cast("Clean Shot", r => ActionManager.LastSpell.Name == "Slug Shot"),
                        Spell.Cast("Slug Shot", r => ActionManager.LastSpell.Name == "Split Shot"),
                        Spell.Cast("Split Shot"),


                        Spell.Cast("Gauss Round"),
                        Spell.Cast("Wildfire",r=> ActionResourceManager.Machinist.Heat >= 50 && Hypercharge.Cooldown.TotalSeconds <= 2),
                        Spell.Cast("Hypercharge"),
                        Spell.Cast("Rook Autoturret"),
                        Spell.Cast("Barrel Stabilizer", r=> ActionResourceManager.Machinist.Heat <= 50)
                        
                        )));
        }

    }

}
