﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Runtime.InteropServices;
using System.Text;
using System.Windows.Forms;
using ff14bot;
using ff14bot.Behavior;
using ff14bot.Enums;
using ff14bot.Helpers;
using ff14bot.Managers;
using ff14bot.Navigation;
using ff14bot.Objects;
using Kupo.Helpers;
using TreeSharp;
using Action = TreeSharp.Action;

namespace Kupo.Rotations
{
    public class Reaper : KupoRoutine
    {
        //DEVELOPERS REPLACE GetType().Name WITH YOUR CR'S NAME.
        public override string Name
        {
            get { return "Kupo [" + GetType().Name + "]"; }
        }

        public override float PullRange
        {
            get { return 2.5f; }
        }

        public override ClassJobType[] Class
        {
            get { return new[] { ClassJobType.Reaper }; }
        }

        [Behavior(BehaviorType.Heal)]
        public Composite CreateBasicHeal()
        {
            return new PrioritySelector(
                Spell.Cast("Bloodbath", r => Core.Player.CurrentHealthPercent < 60, r => Core.Player),
                Spell.Cast("Second Wind", r => Core.Player.CurrentHealthPercent < 40, r => Core.Player)
            );
        }

        [Behavior(BehaviorType.CombatBuffs)]
        public Composite CreateBasicCombatBuffs()
        {
            return new PrioritySelector(SummonChocobo());
        }

        [Behavior(BehaviorType.PreCombatBuffs)]
        public Composite CreateBasicPreCombatBuffs()
        {
            return SummonChocobo();
        }

        [Behavior(BehaviorType.Rest)]
        public Composite CreateBasicRest()
        {
            return DefaultRestBehavior(r => Core.Player.CurrentTPPercent);
        }

        [Behavior(BehaviorType.Pull)]
        public Composite CreateBasicPull()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null,
                    new PrioritySelector(
                        new Decorator(ctx => !RoutineManager.IsAnyDisallowed(CapabilityFlags.Movement), new PrioritySelector(
                        CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
                        CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, ctx => Core.Player.CombatReach + PullRange + (ctx as GameObject).CombatReach, true, "Moving to unit")
                        )),

                        Spell.Apply(r => "Shadow of Death", msLeft: 5000, auraName: "Death's Design"),
                        Spell.PullCast(r=>"Infernal Slice", r => ActionManager.LastSpell.Name == "Waxing Slice"),
                        Spell.PullCast(r=>"Waxing Slice", r => ActionManager.LastSpell.Name == "Slice"),
                        Spell.PullCast("Slice")
            )));
        }


        public static int _SoulReaver
        {
            get
            {
                var aura = Core.Me.GetAuraByName("Soul Reaver");

                if (aura == null)
                    return 0;

                return (int)aura.Value;
            }
        }

        public static string _GibbetOrGallows
        {
            get
            {
                var aura = Core.Me.GetAuraByName("Enhanced Gallows");

                if (aura != null)
                    return "Gallows";

                return "Gibbet";
            }
        }


        private FrameCachedValue<int> SoulReaver = new FrameCachedValue<int>(() => _SoulReaver);
        private FrameCachedObject<string> GibbetOrGallows = new FrameCachedObject<string>(() => _GibbetOrGallows);

        [Behavior(BehaviorType.Combat)]
        public Composite CreateBasicCombat()
        {
            return new PrioritySelector(ctx => Core.Player.CurrentTarget as BattleCharacter,
                new Decorator(ctx => ctx != null,
                    new PrioritySelector(
                        new Decorator(ctx => !RoutineManager.IsAnyDisallowed(CapabilityFlags.Movement), new PrioritySelector(
                        CommonBehaviors.MoveToLos(ctx => ctx as GameObject),
                        CommonBehaviors.MoveAndStop(ctx => (ctx as GameObject).Location, ctx => Core.Player.CombatReach + PullRange + (ctx as GameObject).CombatReach, true, "Moving to unit")
                        )),



                        Spell.Cast(r=>GibbetOrGallows.Value),
                        Spell.Cast("Blood Stalk"),
                        Spell.Cast("Soul Slice"),

                        Spell.Apply(r=>"Shadow of Death",msLeft:5000,auraName:"Death's Design"),
                        Spell.Cast("Infernal Slice", r => ActionManager.LastSpell.Name == "Waxing Slice"),
                        Spell.Cast("Waxing Slice", r => ActionManager.LastSpell.Name == "Slice"),
                        Spell.Cast("Slice")
                        )));
        }
    }
}